@@ -10,6 +10,7 @@
 #include <assert.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <errno.h>
 
 #ifdef HAVE_EVIL
 # include <Evil.h>
@@ -24,8 +25,6 @@
 #endif
 
 #ifdef BUILD_ASYNC_PRELOAD
-#include <pthread.h>
-
 typedef struct _Evas_Cache_Preload Evas_Cache_Preload;
 
 struct _Evas_Cache_Preload
@@ -34,23 +33,12 @@ struct _Evas_Cache_Preload
    Image_Entry *ie;
 };
 
-static Eina_Inlist *preload = NULL;
-static Image_Entry *current = NULL;
-static Eina_List   *pending = NULL;
+static LK(engine_lock) = PTHREAD_MUTEX_INITIALIZER;
+static LK(wakeup) = PTHREAD_MUTEX_INITIALIZER;
 
-static pthread_cond_t cond_done = PTHREAD_COND_INITIALIZER;
-static pthread_cond_t cond_new = PTHREAD_COND_INITIALIZER;
-static pthread_mutex_t mutex_new = PTHREAD_MUTEX_INITIALIZER;
-static pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER;
-static pthread_mutex_t mutex_pending = PTHREAD_MUTEX_INITIALIZER;
-static pthread_mutex_t mutex_surface_alloc = PTHREAD_MUTEX_INITIALIZER;
-static pthread_t tid = 0;
+static pthread_cond_t cond_wakeup = PTHREAD_COND_INITIALIZER;
 
-static Eina_Bool running = EINA_FALSE;
-
-static void *_evas_cache_background_load(void *);
-static void _evas_cache_image_entry_clear_preloaders(Image_Entry *ie);
-static int _evas_cache_image_entry_preload_remove(Image_Entry *ie, const void *target);
+static void _evas_cache_image_entry_preload_remove(Image_Entry *ie, const void *target);
 #endif
 
 #define FREESTRC(Var)              \
@@ -70,7 +58,13 @@ _evas_cache_image_make_dirty(Evas_Cache_Image *cache,
    im->flags.dirty = 1;
    im->flags.activ = 0;
    im->flags.lru_nodata = 0;
+#ifdef EVAS_FRAME_QUEUING
+   LKL(cache->lock);
+#endif
    cache->dirty = eina_inlist_prepend(cache->dirty, EINA_INLIST_GET(im));
+#ifdef EVAS_FRAME_QUEUING
+   LKU(cache->lock);
+#endif
 
    if (im->cache_key)
      {
@@ -84,16 +78,7 @@ _evas_cache_image_make_activ(Evas_Cache_Image *cache,
                              Image_Entry *im,
                              const char *key)
 {
-#ifdef BUILD_ASYNC_PRELOAD
-   pthread_mutex_lock(&mutex);
-   if (im->flags.pending)
-     {
-        im->flags.pending = 0;
-        pending = eina_list_remove(pending, im);
-     }
-   pthread_mutex_unlock(&mutex);
-#endif
-
+   /* FIXME: Handle case when image is being processed anyway and don't do a double decode. */
    im->cache_key = key;
    if (key != NULL)
      {
@@ -101,7 +86,13 @@ _evas_cache_image_make_activ(Evas_Cache_Image *cache,
         im->flags.activ = 1;
         im->flags.lru_nodata = 0;
         im->flags.dirty = 0;
+#ifdef EVAS_FRAME_QUEUING
+        LKL(cache->lock);
+#endif
 	eina_hash_direct_add(cache->activ, key, im);
+#ifdef EVAS_FRAME_QUEUING
+        LKU(cache->lock);
+#endif
      }
    else
      {
@@ -119,9 +110,15 @@ _evas_cache_image_make_inactiv(Evas_Cache_Image *cache,
 	im->flags.activ = 0;
 	im->flags.dirty = 0;
 	im->flags.cached = 1;
+#ifdef EVAS_FRAME_QUEUING
+        LKL(cache->lock);
+#endif
 	eina_hash_direct_add(cache->inactiv, key, im);
 	cache->lru = eina_inlist_prepend(cache->lru, EINA_INLIST_GET(im));
 	cache->usage += cache->func.mem_size_get(im);
+#ifdef EVAS_FRAME_QUEUING
+        LKU(cache->lock);
+#endif
      }
    else
      {
@@ -136,9 +133,15 @@ _evas_cache_image_remove_lru_nodata(Evas_Cache_Image *cache,
    if (im->flags.lru_nodata)
      {
         im->flags.lru_nodata = 0;
+#ifdef EVAS_FRAME_QUEUING
+        LKL(cache->lock);
+#endif
         cache->lru_nodata = eina_inlist_remove(cache->lru_nodata, EINA_INLIST_GET(im));
         cache->usage -= cache->func.mem_size_get(im);
-     }
+#ifdef EVAS_FRAME_QUEUING
+        LKU(cache->lock);
+#endif
+    }
 }
 
 static void
@@ -147,8 +150,14 @@ _evas_cache_image_activ_lru_nodata(Evas_Cache_Image *cache,
 {
    im->flags.need_data = 0;
    im->flags.lru_nodata = 1;
+#ifdef EVAS_FRAME_QUEUING
+   LKL(cache->lock);
+#endif
    cache->lru_nodata = eina_inlist_prepend(cache->lru_nodata, EINA_INLIST_GET(im));
    cache->usage += cache->func.mem_size_get(im);
+#ifdef EVAS_FRAME_QUEUING
+   LKU(cache->lock);
+#endif
 }
 
 static void
@@ -159,11 +168,20 @@ _evas_cache_image_remove_activ(Evas_Cache_Image *cache,
      {
         if (ie->flags.activ)
           {
+#ifdef EVAS_FRAME_QUEUING
+             LKL(cache->lock);
+#endif
 	     eina_hash_del(cache->activ, ie->cache_key, ie);
+#ifdef EVAS_FRAME_QUEUING
+             LKU(cache->lock);
+#endif
              _evas_cache_image_remove_lru_nodata(cache, ie);
           }
         else
           {
+#ifdef EVAS_FRAME_QUEUING
+             LKL(cache->lock);
+#endif
              if (ie->flags.dirty)
                {
  		  cache->dirty = eina_inlist_remove(cache->dirty, EINA_INLIST_GET(ie));
@@ -174,6 +192,9 @@ _evas_cache_image_remove_activ(Evas_Cache_Image *cache,
                   cache->lru = eina_inlist_remove(cache->lru, EINA_INLIST_GET(ie));
                   cache->usage -= cache->func.mem_size_get(ie);
                }
+#ifdef EVAS_FRAME_QUEUING
+             LKU(cache->lock);
+#endif
           }
         ie->flags.cached = 0;
         ie->flags.dirty = 0;
@@ -184,13 +205,23 @@ _evas_cache_image_remove_activ(Evas_Cache_Image *cache,
 static void
 _evas_cache_image_entry_delete(Evas_Cache_Image *cache, Image_Entry *ie)
 {
-   if (!ie) return ;
+   if (!ie)
+   	return ;
 
    if (cache->func.debug)
      cache->func.debug("deleting", ie);
 
 #ifdef BUILD_ASYNC_PRELOAD
-   _evas_cache_image_entry_preload_remove(ie, NULL);
+   if (ie->flags.delete_me == 1)
+     return ;
+
+   if (ie->preload)
+     {
+	ie->flags.delete_me = 1;
+
+	_evas_cache_image_entry_preload_remove(ie, NULL);
+	return ;
+     }
 #endif
 
    cache->func.destructor(ie);
@@ -213,6 +244,9 @@ _evas_cache_image_entry_delete(Evas_Cache_Image *cache, Image_Entry *ie)
 #ifdef BUILD_ASYNC_PRELOAD
    LKD(ie->lock);
 #endif
+#ifdef EVAS_FRAME_QUEUING
+   LKD(ie->lock_references);
+#endif
 
    cache->func.dealloc(ie);
 }
@@ -231,7 +265,10 @@ _evas_cache_image_entry_new(Evas_Cache_Image *cache,
 
    ie = cache->func.alloc();
    if (!ie)
-     return NULL;
+     {
+	*error = EVAS_LOAD_ERROR_RESOURCE_ALLOCATION_FAILED;
+	return NULL;
+     }
 
    cache_key = hkey ? eina_stringshare_add(hkey) : NULL;
 
@@ -246,6 +283,9 @@ _evas_cache_image_entry_new(Evas_Cache_Image *cache,
    ie->allocated.w = 0;
    ie->allocated.h = 0;
 
+#ifdef EVAS_FRAME_QUEUING
+   LKI(ie->lock_references);
+#endif
    ie->references = 0;
    ie->cache = cache;
 
@@ -268,6 +308,8 @@ _evas_cache_image_entry_new(Evas_Cache_Image *cache,
 #ifdef BUILD_ASYNC_PRELOAD
    LKI(ie->lock);
    ie->targets = NULL;
+   ie->preload = NULL;
+   ie->flags.delete_me = 0;
 #endif
 
    if (lo)
@@ -276,7 +318,7 @@ _evas_cache_image_entry_new(Evas_Cache_Image *cache,
    if (file)
      {
         *error = cache->func.constructor(ie);
-        if (*error != 0)
+        if (*error != EVAS_LOAD_ERROR_NONE)
           {
              _evas_cache_image_entry_delete(cache, ie);
              return NULL;
@@ -322,52 +364,105 @@ _evas_cache_image_entry_surface_alloc(Evas_Cache_Image *cache,
    hmin = h > 0 ? h : 1;
 
 #ifdef BUILD_ASYNC_PRELOAD
-   pthread_mutex_lock(&mutex_surface_alloc);
+   LKL(engine_lock);
 #endif
    _evas_cache_image_entry_surface_alloc__locked(cache, ie, wmin, hmin);
 #ifdef BUILD_ASYNC_PRELOAD
-   pthread_mutex_unlock(&mutex_surface_alloc);
+   LKU(engine_lock);
 #endif
 }
 
 #ifdef BUILD_ASYNC_PRELOAD
 static void
-_evas_cache_image_async_call_process(void *obj, Evas_Callback_Type type, void *data)
+_evas_cache_image_async_heavy(void *data)
 {
-   Image_Entry *ie = (Image_Entry *) obj;
+   Evas_Cache_Image *cache;
+   Image_Entry *current;
+   int error;
+   int pchannel;
 
-   ie->flags.in_pipe = 0;
+   current = data;
+
+   LKL(current->lock);
+   pchannel = current->channel;
+   current->channel++;
+   cache = current->cache;
+
+   if (!current->flags.loaded && ((Evas_Image_Load_Func*) current->info.module)->threadable)
+     {
+	error = cache->func.load(current);
+	if (cache->func.debug)
+	  cache->func.debug("load", current);
+	if (error != EVAS_LOAD_ERROR_NONE)
+	  {
+	     current->flags.loaded = 0;
+	     _evas_cache_image_entry_surface_alloc(cache, current,
+						   current->w, current->h);
+	  }
+	else
+	  current->flags.loaded = 1;
+     }
+
+   current->channel = pchannel;
+   LKU(current->lock);
+}
+
+static void
+_evas_cache_image_async_end(void *data)
+{
+   Image_Entry *ie = (Image_Entry *) data;
+   Evas_Cache_Target *tmp;
+
+   ie->cache->preload = eina_list_remove(ie->cache->preload, ie);
+   ie->cache->pending = eina_list_remove(ie->cache->pending, ie);
+
+   ie->preload = NULL;
+   ie->flags.preload_done = ie->flags.loaded;
    while (ie->targets)
      {
-	Evas_Cache_Target *tmp = ie->targets;
+	tmp = ie->targets;
 
-	evas_object_event_callback_call((Evas_Object*) tmp->target, EVAS_CALLBACK_IMAGE_PRELOADED, NULL);
+	evas_object_inform_call_image_preloaded((Evas_Object*) tmp->target);
 	ie->targets = (Evas_Cache_Target*) eina_inlist_remove(EINA_INLIST_GET(ie->targets), EINA_INLIST_GET(ie->targets));
 	free(tmp);
      }
 }
 
 static void
-_evas_cache_image_entry_clear_preloaders(Image_Entry *ie)
+_evas_cache_image_async_cancel(void *data)
 {
-   ie->flags.preload = 0;
-   ie->flags.in_pipe = 0;
-   evas_async_target_del(ie);
-}
+   Evas_Cache_Image *cache = NULL;
+   Image_Entry *ie = (Image_Entry *) data;
 
-static void
-_evas_cache_image_async_call__locked(Image_Entry *im)
-{
-   evas_async_events_put(im, EVAS_CALLBACK_IMAGE_PRELOADED, NULL,
-			 _evas_cache_image_async_call_process);
-}
+   ie->preload = NULL;
+   ie->cache->pending = eina_list_remove(ie->cache->pending, ie);
 
-static void
-_evas_cache_image_async_call(Image_Entry *im)
-{
-   pthread_mutex_lock(&mutex);
-   _evas_cache_image_async_call__locked(im);
-   pthread_mutex_unlock(&mutex);
+   if (ie->flags.delete_me || ie->flags.dirty)
+     {
+	ie->flags.delete_me = 0;
+	_evas_cache_image_entry_delete(ie->cache, ie);
+
+	return ;
+     }
+
+   if (ie->flags.loaded)
+     {
+	_evas_cache_image_async_end(ie);
+     }
+
+#ifdef EVAS_FRAME_QUEUING
+   LKL(ie->lock_references);
+#endif
+   if (ie->references == 0)
+     {
+	_evas_cache_image_remove_activ(ie->cache, ie);
+	_evas_cache_image_make_inactiv(ie->cache, ie, ie->cache_key);
+        cache = ie->cache;
+     }
+#ifdef EVAS_FRAME_QUEUING
+   LKU(ie->lock_references);
+#endif
+   if (cache) evas_cache_image_flush(cache);
 }
 
 static int
@@ -375,66 +470,33 @@ _evas_cache_image_entry_preload_add(Image_Entry *ie,
 				    const void *target)
 {
    Evas_Cache_Target *tg;
-   int ret = 0;
 
-   pthread_mutex_lock(&mutex);
+   if (ie->flags.preload_done) return 0;
 
-   if (!ie->flags.loaded)
+   tg = malloc(sizeof (Evas_Cache_Target));
+   if (!tg) return 0;
+
+   tg->target = target;
+
+   ie->targets = (Evas_Cache_Target*) eina_inlist_append(EINA_INLIST_GET(ie->targets), EINA_INLIST_GET(tg));
+
+   if (!ie->preload)
      {
-	tg = malloc(sizeof (Evas_Cache_Target));
-	if (!tg) goto end;
+        ie->cache->preload = eina_list_append(ie->cache->preload, ie);
+        ie->flags.pending = 0;
 
-	tg->target = target;
-	ie->targets = (Evas_Cache_Target*) eina_inlist_append(EINA_INLIST_GET(ie->targets), EINA_INLIST_GET(tg));
-
-	if (!ie->flags.preload)
-	  {
-	     Evas_Cache_Preload *tmp;
-
-	     tmp = malloc(sizeof (Evas_Cache_Preload));
-	     if (!tmp) goto end;
-
-	     tmp->ie = ie;
-	     preload = eina_inlist_append(preload, EINA_INLIST_GET(tmp));
-
-	     ie->flags.preload = 1;
-             if (ie->flags.pending)
-               {
-                  ie->flags.pending = 0;
-                  pending = eina_list_remove(pending, ie);
-               }
-
-	     if (!running)
-	       {
-		  if (tid)
-		    {
-		       running = EINA_TRUE;
-		       pthread_cond_signal(&cond_new);
-		    }
-		  else
-		    {
-		       if (pthread_create(&tid, NULL, _evas_cache_background_load, NULL) == 0)
-			 running = EINA_TRUE;
-		    }
-	       }
-
-	     ret = 2;
-	  }
-	ret = 1;
+        ie->preload = evas_preload_thread_run(_evas_cache_image_async_heavy,
+                                              _evas_cache_image_async_end,
+                                              _evas_cache_image_async_cancel,
+                                              ie);
      }
 
- end:
-   pthread_mutex_unlock(&mutex);
-
-   return ret;
+   return 1;
 }
 
-static int
+static void
 _evas_cache_image_entry_preload_remove(Image_Entry *ie, const void *target)
 {
-   int ret = 0;
-
-   pthread_mutex_lock(&mutex);
    if (target)
      {
 	Evas_Cache_Target *tg;
@@ -450,140 +512,28 @@ _evas_cache_image_entry_preload_remove(Image_Entry *ie, const void *target)
 	       }
 	  }
      }
-
-   if (ie->flags.in_pipe)
+   else
      {
-	if (!ie->targets)
-	  _evas_cache_image_entry_clear_preloaders(ie);
-     }
-   else if (ie->flags.preload)
-     {
-	if (current == ie)
+	Evas_Cache_Target *tg;
+
+	while (ie->targets)
 	  {
-// dont wait. simply handle "ie->flags.preload" nicely
-//		  /* Wait until ie is processed. */
-//		  pthread_cond_wait(&cond_done, &mutex);
-	  }
-	else
-	  {
-	     Evas_Cache_Preload *l;
+	     tg = ie->targets;
 
-	     EINA_INLIST_FOREACH(preload, l)
-	       {
-		  if (l->ie == ie)
-		    {
-		       if (target)
-			 {
-			    // FIXME: No callback when we cancel only for one target ?
-			    if (!ie->targets)
-			      _evas_cache_image_entry_clear_preloaders(ie);
-			 }
-		       else
-			 {
-			    _evas_cache_image_async_call__locked(ie);
-			 }
-
-		       if (!ie->targets)
-			 {
-			    ie->flags.preload = 0;
-			    preload = eina_inlist_remove(preload,
-							 EINA_INLIST_GET(l));
-			    free(l);
-			 }
-
-		       break;
-		    }
-	       }
-	     ret = 1;
+	     ie->targets = (Evas_Cache_Target*) eina_inlist_remove(EINA_INLIST_GET(ie->targets), EINA_INLIST_GET(tg));
+	     free(tg);
 	  }
      }
-   pthread_mutex_unlock(&mutex);
 
-   return ret;
-}
-
-static void*
-_evas_cache_background_load(void *data)
-{
-   (void) data;
-   
-   restart:
-   while (preload)
+   if (ie->targets == NULL && ie->preload && !ie->flags.pending)
      {
-	pthread_mutex_lock(&mutex);
-	if (preload)
-	  {
-	     Evas_Cache_Preload *tmp = (Evas_Cache_Preload*) preload;
-             
-	     current = tmp->ie;
-	     preload = eina_inlist_remove(preload, preload);
+	ie->cache->preload = eina_list_remove(ie->cache->preload, ie);
+	ie->cache->pending = eina_list_append(ie->cache->pending, ie);
 
-	     free(tmp);
-	  }
+	ie->flags.pending = 1;
 
-	pthread_mutex_unlock(&mutex);
-
-	if (current)
-	  {
-	     Evas_Cache_Image *cache;
-	     int error;
-             int pchannel;
-
-	     LKL(current->lock);
-             pchannel = current->channel;
-             current->channel++;
-	     cache = current->cache;
-
-             if (!current->flags.loaded)
-               {
-                  error = cache->func.load(current);
-                  if (cache->func.debug)
-                    cache->func.debug("load", current);
-                  if (error)
-                    {
-                       _evas_cache_image_entry_surface_alloc
-                         (cache, current, current->w, current->h);
-                       current->flags.loaded = 0;
-                    }
-                  else
-                    current->flags.loaded = 1;
-               }
-             
-             pthread_mutex_lock(&mutex);
-             pthread_mutex_lock(&mutex_pending);
-	     current->flags.preload = 0;
-	     current->flags.in_pipe = 1;
-             current->channel = pchannel;
-	     LKU(current->lock);
-             pthread_mutex_unlock(&mutex_pending);
-             pthread_mutex_unlock(&mutex);
-             
-	     _evas_cache_image_async_call(current);
-             
-             pthread_mutex_lock(&mutex);
-	     current = NULL;
-             pthread_mutex_unlock(&mutex);
-	  }
-
-	pthread_cond_signal(&cond_done);
+	evas_preload_thread_cancel(ie->preload);
      }
-
-   pthread_mutex_lock(&mutex);
-   if (preload)
-     {
-	pthread_mutex_unlock(&mutex);
-	goto restart;
-     }
-
-   running = EINA_FALSE;
-   pthread_mutex_unlock(&mutex);
-
-   pthread_mutex_lock(&mutex_new);
-   pthread_cond_wait(&cond_new, &mutex_new);
-   pthread_mutex_unlock(&mutex_new);
-   goto restart;
-
-   return NULL;
 }
 #endif
 
@@ -607,9 +557,21 @@ EAPI void
 evas_cache_image_set(Evas_Cache_Image *cache, int limit)
 {
    assert(cache != NULL);
-   if (cache->limit == limit) return;
+#ifdef EVAS_FRAME_QUEUING
+   LKL(cache->lock);
+#endif
+   if (cache->limit == limit)
+     {
+#ifdef EVAS_FRAME_QUEUING
+        LKU(cache->lock);
+#endif
+        return;
+     }
    cache->limit = limit;
    evas_cache_image_flush(cache);
+#ifdef EVAS_FRAME_QUEUING
+   LKU(cache->lock);
+#endif
 }
 
 EAPI Evas_Cache_Image *
@@ -634,6 +596,13 @@ evas_cache_image_init(const Evas_Cache_Image_Func *cb)
 
    new->references = 1;
 
+   new->preload = NULL;
+   new->pending = NULL;
+
+#ifdef EVAS_FRAME_QUEUING
+   LKI(new->lock);
+#endif
+
    return new;
 }
 
@@ -650,37 +619,32 @@ _evas_cache_image_free_cb(__UNUSED__ const Eina_Hash *hash, __UNUSED__ const voi
 EAPI void
 evas_cache_image_shutdown(Evas_Cache_Image *cache)
 {
-   Image_Entry  *im;
    Eina_List *delete_list;
+   Image_Entry  *im;
 
    assert(cache != NULL);
+#ifdef EVAS_FRAME_QUEUING
+   LKL(cache->lock);
+#endif
    cache->references--;
 
    if (cache->references > 0)
-     return ;
-
-#ifdef BUILD_ASYNC_PRELOAD
-   pthread_mutex_lock(&mutex);
-
-   Eina_Inlist *l, *l_next;
-   for (l = preload; l != NULL; l = l_next)
      {
-	Evas_Cache_Preload *tmp = (Evas_Cache_Preload *)l;
-	Image_Entry *ie = tmp->ie;
-
-	l_next = l->next;
-
-	if (ie->cache != cache)
-	  continue;
-
-	preload = eina_inlist_remove(preload, l);
-	_evas_cache_image_entry_clear_preloaders(ie);
-	free(l);
+#ifdef EVAS_FRAME_QUEUING
+        LKU(cache->lock);
+#endif
+        return ;
      }
 
-   if (current && current->cache == cache)
-     _evas_cache_image_entry_clear_preloaders(current);
-   pthread_mutex_unlock(&mutex);
+#ifdef BUILD_ASYNC_PRELOAD
+   EINA_LIST_FREE(cache->preload, im)
+     {
+	/* By doing that we are protecting us from destroying image when the cache is no longuer available. */
+	im->flags.delete_me = 1;
+	_evas_cache_image_entry_preload_remove(im, NULL);
+     }
+
+   evas_async_events_process();
 #endif
 
    while (cache->lru)
@@ -711,9 +675,27 @@ evas_cache_image_shutdown(Evas_Cache_Image *cache)
 	delete_list = eina_list_remove_list(delete_list, delete_list);
      }
 
+#ifdef BUILD_ASYNC_PRELOAD
+   /* Now wait for all pending image to die */
+   while (cache->pending)
+     {
+	evas_async_events_process();
+
+	LKL(wakeup);
+	if (cache->pending)
+	  pthread_cond_wait(&cond_wakeup, &wakeup);
+	LKU(wakeup);
+     }
+#endif
+
    eina_hash_free(cache->activ);
    eina_hash_free(cache->inactiv);
 
+#ifdef EVAS_FRAME_QUEUING
+   LKU(cache->lock);
+   LKD(cache->lock);
+#endif
+
    free(cache);
 }
 
@@ -734,8 +716,11 @@ evas_cache_image_request(Evas_Cache_Image *cache, const char *file, const char *
 
    assert(cache != NULL);
 
-   if (!file && !key) return NULL;
-   if (!file) return NULL;
+   if ((!file) || ((!file) && (!key)))
+     {
+	*error = EVAS_LOAD_ERROR_GENERIC;
+	return NULL;
+     }
 
    file_length = strlen(file);
    key_length = key ? strlen(key) : 6;
@@ -812,7 +797,13 @@ evas_cache_image_request(Evas_Cache_Image *cache, const char *file, const char *
 
    hkey[size] = '\0';
 
+#ifdef EVAS_FRAME_QUEUING
+   LKL(cache->lock);
+#endif
    im = eina_hash_find(cache->activ, hkey);
+#ifdef EVAS_FRAME_QUEUING
+   LKU(cache->lock);
+#endif
    if (im)
      {
         time_t  t;
@@ -823,7 +814,7 @@ evas_cache_image_request(Evas_Cache_Image *cache, const char *file, const char *
         if ((t - im->laststat) > STAT_GAP)
           {
              stat_done = 1;
-             if (stat(file, &st) < 0) goto on_error;
+             if (stat(file, &st) < 0) goto on_stat_error;
 
              im->laststat = t;
              if (st.st_mtime != im->timestamp) ok = 0;
@@ -834,7 +825,13 @@ evas_cache_image_request(Evas_Cache_Image *cache, const char *file, const char *
 	_evas_cache_image_make_dirty(cache, im);
      }
 
+#ifdef EVAS_FRAME_QUEUING
+   LKL(cache->lock);
+#endif
    im = eina_hash_find(cache->inactiv, hkey);
+#ifdef EVAS_FRAME_QUEUING
+   LKU(cache->lock);
+#endif
    if (im)
      {
         int     ok;
@@ -848,7 +845,7 @@ evas_cache_image_request(Evas_Cache_Image *cache, const char *file, const char *
              if ((t - im->laststat) > STAT_GAP)
                {
                   stat_done = 1;
-                  if (stat(file, &st) < 0) goto on_error;
+                  if (stat(file, &st) < 0) goto on_stat_error;
 
                   im->laststat = t;
                   if (st.st_mtime != im->timestamp) ok = 0;
@@ -869,7 +866,7 @@ evas_cache_image_request(Evas_Cache_Image *cache, const char *file, const char *
 
    if (!stat_done)
      {
-        if (stat(file, &st) < 0) return NULL;
+        if (stat(file, &st) < 0) goto on_stat_error;
      }
 
    im = _evas_cache_image_entry_new(cache, hkey, st.st_mtime, file, key, lo, error);
@@ -879,86 +876,82 @@ evas_cache_image_request(Evas_Cache_Image *cache, const char *file, const char *
      cache->func.debug("request", im);
 
  on_ok:
-   *error = 0;
+   *error = EVAS_LOAD_ERROR_NONE;
+#ifdef EVAS_FRAME_QUEUING
+   LKL(im->lock_references);
+#endif
    im->references++;
    if (im->references > 1 && im->flags.lru_nodata)
      _evas_cache_image_remove_lru_nodata(cache, im);
+#ifdef EVAS_FRAME_QUEUING
+   LKU(im->lock_references);
+#endif
 
    return im;
 
- on_error:
-   _evas_cache_image_entry_delete(cache, im);
+ on_stat_error:
+#ifndef _WIN32
+   if ((errno == ENOENT) || (errno == ENOTDIR) ||
+       (errno == ENAMETOOLONG) || (errno == ELOOP))
+#else
+   if (errno == ENOENT)
+#endif
+     *error = EVAS_LOAD_ERROR_DOES_NOT_EXIST;
+#ifndef _WIN32
+   else if ((errno == ENOMEM) || (errno == EOVERFLOW))
+#else
+   else if (errno == ENOMEM)
+#endif
+     *error = EVAS_LOAD_ERROR_RESOURCE_ALLOCATION_FAILED;
+   else if (errno == EACCES)
+     *error = EVAS_LOAD_ERROR_PERMISSION_DENIED;
+   else
+     *error = EVAS_LOAD_ERROR_GENERIC;
+
+   if (im) _evas_cache_image_entry_delete(cache, im);
    return NULL;
 }
 
-EAPI void
-evas_cache_pending_process(void)
-{
-   Image_Entry *im;
-   
-#ifdef BUILD_ASYNC_PRELOAD
-   pthread_mutex_lock(&mutex_pending);
-   EINA_LIST_FREE(pending, im)
-     {
-        Evas_Cache_Image *cache = im->cache;
-
-        if (!im->flags.pending) continue;
-        im->flags.pending = 0;
-        if (im->flags.preload) continue;
-        
-        if (im->flags.activ)
-          {
-             if (im->flags.dirty)
-               {
-                  _evas_cache_image_entry_delete(cache, im);
-               }
-             else
-               {
-                  _evas_cache_image_remove_activ(cache, im);
-                  _evas_cache_image_make_inactiv(cache, im, im->cache_key);
-                  evas_cache_image_flush(cache);
-               }
-          }
-        else if (im->flags.cached)
-          {
-             evas_cache_image_flush(cache);
-          }
-     }
-   pthread_mutex_unlock(&mutex_pending);
-#endif   
-}
-
 EAPI void
 evas_cache_image_drop(Image_Entry *im)
 {
    Evas_Cache_Image    *cache;
+   int                              references;
 
    assert(im);
    assert(im->cache);
 
+#ifdef EVAS_FRAME_QUEUING
+   LKL(im->lock_references);
+#endif
    im->references--;
+   references = im->references;
+#ifdef EVAS_FRAME_QUEUING
+   LKU(im->lock_references);
+#endif
+   
    cache = im->cache;
 
-   if (im->references == 0)
+   if (references == 0)
      {
+#ifdef EVAS_FRAME_QUEUING
+        LKL(((RGBA_Image *)im)->ref_fq_add);
+        LKL(((RGBA_Image *)im)->ref_fq_del);
+        if (((RGBA_Image *)im)->ref_fq[0] != ((RGBA_Image *)im)->ref_fq[1])
+          {
+             LKU(((RGBA_Image *)im)->ref_fq_add);
+             LKU(((RGBA_Image *)im)->ref_fq_del);
+             return;
+          }
+	LKU(((RGBA_Image *)im)->ref_fq_add);
+	LKU(((RGBA_Image *)im)->ref_fq_del);
+#endif
+
 #ifdef BUILD_ASYNC_PRELOAD
-	if (!im->flags.pending)
+	if (im->preload)
 	  {
-	     pthread_mutex_lock(&mutex);
-	     if (im->flags.preload || im->flags.in_pipe)
-	       {
-		  pthread_mutex_unlock(&mutex);
-		  _evas_cache_image_entry_preload_remove(im, NULL);
-		  pthread_mutex_lock(&mutex_pending);
-		  if (!im->flags.pending)
-		    {
-		       im->flags.pending = 1;
-		       pending = eina_list_append(pending, im);
-		    }
-		  pthread_mutex_unlock(&mutex_pending);
-		  return;
-	       }
-	     pthread_mutex_unlock(&mutex);
+	     _evas_cache_image_entry_preload_remove(im, NULL);
+	     return ;
 	  }
 #endif
 
@@ -967,7 +960,7 @@ evas_cache_image_drop(Image_Entry *im)
 	     _evas_cache_image_entry_delete(cache, im);
 	     return;
 	  }
-        
+
         _evas_cache_image_remove_activ(cache, im);
 	_evas_cache_image_make_inactiv(cache, im, im->cache_key);
 	evas_cache_image_flush(cache);
@@ -978,13 +971,22 @@ EAPI void
 evas_cache_image_data_not_needed(Image_Entry *im)
 {
    Evas_Cache_Image    *cache;
-
+   int   references;
+   
    assert(im);
    assert(im->cache);
 
    cache = im->cache;
 
-   if (im->references > 1) return ;
+#ifdef EVAS_FRAME_QUEUING
+   LKL(im->lock_references);
+#endif
+   references = im->references;
+#ifdef EVAS_FRAME_QUEUING
+   LKU(im->lock_references);
+#endif
+
+   if (references > 1) return ;
    if (im->flags.dirty || !im->flags.need_data) return ;
 
    _evas_cache_image_activ_lru_nodata(cache, im);
@@ -995,6 +997,7 @@ evas_cache_image_dirty(Image_Entry *im, int x, int y, int w, int h)
 {
    Image_Entry          *im_dirty = im;
    Evas_Cache_Image     *cache;
+   int                           references;
 
    assert(im);
    assert(im->cache);
@@ -1002,9 +1005,17 @@ evas_cache_image_dirty(Image_Entry *im, int x, int y, int w, int h)
    cache = im->cache;
    if (!(im->flags.dirty))
      {
+#ifdef EVAS_FRAME_QUEUING
+   LKL(im->lock_references);
+#endif
+   references = im->references;
+#ifdef EVAS_FRAME_QUEUING
+   LKU(im->lock_references);
+#endif
+
 #ifndef EVAS_CSERVE
 // if ref 1 also copy if using shared cache as its read-only
-        if (im->references == 1) im_dirty = im;
+        if (references == 1) im_dirty = im;
         else
 #endif
           {
@@ -1033,7 +1044,13 @@ evas_cache_image_dirty(Image_Entry *im, int x, int y, int w, int h)
 
              if (error != 0) goto on_error;
  */
+#ifdef EVAS_FRAME_QUEUING
+             LKL(im_dirty->lock_references);
+#endif
              im_dirty->references = 1;
+#ifdef EVAS_FRAME_QUEUING
+             LKU(im_dirty->lock_references);
+#endif
 
              evas_cache_image_drop(im);
           }
@@ -1060,12 +1077,21 @@ evas_cache_image_alone(Image_Entry *im)
 {
    Evas_Cache_Image     *cache;
    Image_Entry          *im_dirty = im;
+   int                           references;
 
    assert(im);
    assert(im->cache);
 
    cache = im->cache;
-   if (im->references == 1)
+#ifdef EVAS_FRAME_QUEUING
+   LKL(im->lock_references);
+#endif
+   references = im->references;
+#ifdef EVAS_FRAME_QUEUING
+   LKU(im->lock_references);
+#endif
+
+   if (references == 1)
      {
         if (!(im->flags.dirty))
           {
@@ -1100,7 +1126,13 @@ evas_cache_image_alone(Image_Entry *im)
 
         if (error != 0) goto on_error;
  */
+#ifdef EVAS_FRAME_QUEUING
+   LKL(im_dirty->lock_references);
+#endif
         im_dirty->references = 1;
+#ifdef EVAS_FRAME_QUEUING
+   LKU(im_dirty->lock_references);
+#endif
 
         evas_cache_image_drop(im);
      }
@@ -1137,7 +1169,13 @@ evas_cache_image_copied_data(Evas_Cache_Image *cache, int w, int h, DATA32 *imag
         _evas_cache_image_entry_delete(cache, im);
         return NULL;
      }
+#ifdef EVAS_FRAME_QUEUING
+   LKL(im->lock_references);
+#endif
    im->references = 1;
+#ifdef EVAS_FRAME_QUEUING
+   LKU(im->lock_references);
+#endif
 
    if (cache->func.debug)
      cache->func.debug("copied-data", im);
@@ -1165,7 +1203,13 @@ evas_cache_image_data(Evas_Cache_Image *cache, int w, int h, DATA32 *image_data,
         _evas_cache_image_entry_delete(cache, im);
         return NULL;
      }
+#ifdef EVAS_FRAME_QUEUING
+   LKL(im->lock_references);
+#endif
    im->references = 1;
+#ifdef EVAS_FRAME_QUEUING
+   LKU(im->lock_references);
+#endif
 
    if (cache->func.debug)
      cache->func.debug("data", im);
@@ -1201,7 +1245,13 @@ evas_cache_image_size_set(Image_Entry *im, int w, int h)
 
    assert(im);
    assert(im->cache);
+#ifdef EVAS_FRAME_QUEUING
+   LKL(im->lock_references);
+#endif
    assert(im->references > 0);
+#ifdef EVAS_FRAME_QUEUING
+   LKU(im->lock_references);
+#endif
 
    if ((im->space == EVAS_COLORSPACE_YCBCR422P601_PL) ||
        (im->space == EVAS_COLORSPACE_YCBCR422P709_PL))
@@ -1224,7 +1274,13 @@ evas_cache_image_size_set(Image_Entry *im, int w, int h)
    error = cache->func.size_set(new, im, w, h);
    if (error != 0) goto on_error;
 
+#ifdef EVAS_FRAME_QUEUING
+   LKL(new->lock_references);
+#endif
    new->references = 1;
+#ifdef EVAS_FRAME_QUEUING
+   LKU(new->lock_references);
+#endif
 
    evas_cache_image_drop(im);
 
@@ -1242,8 +1298,11 @@ evas_cache_image_size_set(Image_Entry *im, int w, int h)
 EAPI void
 evas_cache_image_load_data(Image_Entry *im)
 {
-   Evas_Cache_Image    *cache;
-   int                  error, preload;
+#ifdef BUILD_ASYNC_PRELOAD
+   Eina_Bool preload = EINA_FALSE;
+#endif
+   Evas_Cache_Image *cache;
+   int error;
 
    assert(im);
    assert(im->cache);
@@ -1253,44 +1312,121 @@ evas_cache_image_load_data(Image_Entry *im)
      {
 	return;
      }
+
 #ifdef BUILD_ASYNC_PRELOAD
-   pthread_mutex_lock(&mutex);
-   preload = im->flags.preload;
-   /* We check a first time, to prevent useless lock. */
-   if (preload)
+   if (im->preload)
      {
-        if (current == im)
-          {
-             /* Wait until ie is processed. */
-             pthread_cond_wait(&cond_done, &mutex);
-          }
+	preload = EINA_TRUE;
+
+	if (!im->flags.pending)
+	  {
+	     im->cache->preload = eina_list_remove(im->cache->preload, im);
+	     im->cache->pending = eina_list_append(im->cache->pending, im);
+	     im->flags.pending = 1;
+
+	     evas_preload_thread_cancel(im->preload);
+	  }
+
+	evas_async_events_process();
+
+	LKL(wakeup);
+	while (im->preload)
+	  {
+	     pthread_cond_wait(&cond_wakeup, &wakeup);
+	     LKU(wakeup);
+
+	     evas_async_events_process();
+
+	     LKL(wakeup);
+	  }
+	LKU(wakeup);
      }
-   pthread_mutex_unlock(&mutex);
-//   _evas_cache_image_entry_preload_remove(im, NULL);
+
    if (im->flags.loaded) return ;
    LKL(im->lock);
 #endif
 
+   im->flags.in_progress = EINA_TRUE;
    error = cache->func.load(im);
+   im->flags.in_progress = EINA_FALSE;
+
 #ifdef BUILD_ASYNC_PRELOAD
    LKU(im->lock);
 #endif
 
+   im->flags.loaded = 1;
+
    if (cache->func.debug)
      cache->func.debug("load", im);
-#ifdef BUILD_ASYNC_PRELOAD
-   if (preload)
-     _evas_cache_image_async_call(im);
-#endif
 
-   if (error)
+   if (error != EVAS_LOAD_ERROR_NONE)
      {
         _evas_cache_image_entry_surface_alloc(cache, im, im->w, im->h);
         im->flags.loaded = 0;
-        return ;
      }
 
-   im->flags.loaded = 1;
+#ifdef BUILD_ASYNC_PRELOAD
+   if (preload)
+     _evas_cache_image_async_end(im);
+#endif
+}
+
+EAPI void
+evas_cache_image_unload_data(Image_Entry *im)
+{
+   Evas_Cache_Image *cache;
+
+   assert(im);
+   assert(im->cache);
+   cache = im->cache;
+
+   if (im->flags.in_progress) return ;
+
+   evas_cache_image_preload_cancel(im, NULL);
+
+#ifdef BUILD_ASYNC_PRELOAD
+   LKL(im->lock);
+#endif
+   if ((!im->flags.loaded) || (!im->file) ||
+       (!im->info.module) || (im->flags.dirty))
+     {
+#ifdef BUILD_ASYNC_PRELOAD
+        LKU(im->lock);
+#endif
+	return;
+     }
+   evas_common_rgba_image_scalecache_dirty(im);
+   cache->func.destructor(im);
+
+#ifdef BUILD_ASYNC_PRELOAD
+   LKU(im->lock);
+#endif
+}
+
+static Eina_Bool
+_evas_cache_image_unload_cb(__UNUSED__ const Eina_Hash *hash, __UNUSED__ const void *key, void *data, __UNUSED__ void *fdata)
+{
+   evas_cache_image_unload_data(data);
+   return EINA_TRUE;
+}
+
+EAPI void
+evas_cache_image_unload_all(Evas_Cache_Image *cache)
+{
+   Image_Entry *im;
+
+   EINA_INLIST_FOREACH(cache->lru, im) evas_cache_image_unload_data(im);
+   EINA_INLIST_FOREACH(cache->lru_nodata, im) evas_cache_image_unload_data(im);
+   eina_hash_foreach(cache->activ, _evas_cache_image_unload_cb, NULL);
+   eina_hash_foreach(cache->inactiv, _evas_cache_image_unload_cb, NULL);
+}
+
+EAPI Eina_Bool
+evas_cache_image_is_loaded(Image_Entry *im)
+{
+  assert(im); 
+  if (im->flags.loaded) return EINA_TRUE;
+  return EINA_FALSE;
 }
 
 EAPI void
@@ -1304,18 +1440,17 @@ evas_cache_image_preload_data(Image_Entry *im, const void *target)
 
    if (im->flags.loaded)
      {
-	evas_object_event_callback_call((Evas_Object*) target, EVAS_CALLBACK_IMAGE_PRELOADED, NULL);
-	return ;
+	evas_object_inform_call_image_preloaded((Evas_Object *)target);
+	return;
      }
 
    cache = im->cache;
 
    if (!_evas_cache_image_entry_preload_add(im, target))
-     evas_object_event_callback_call((Evas_Object*) target, EVAS_CALLBACK_IMAGE_PRELOADED, NULL);
+     evas_object_inform_call_image_preloaded((Evas_Object *)target);
 #else
    evas_cache_image_load_data(im);
-
-   evas_object_event_callback_call((Evas_Object*) target, EVAS_CALLBACK_IMAGE_PRELOADED, NULL);
+   evas_object_inform_call_image_preloaded((Evas_Object *)target);
 #endif
 }
 
@@ -1323,7 +1458,7 @@ EAPI void
 evas_cache_image_preload_cancel(Image_Entry *im, const void *target)
 {
 #ifdef BUILD_ASYNC_PRELOAD
-   Evas_Cache_Image    *cache;
+   Evas_Cache_Image *cache;
 
    assert(im);
    assert(im->cache);
@@ -1332,17 +1467,8 @@ evas_cache_image_preload_cancel(Image_Entry *im, const void *target)
    if (target == NULL) return ;
 
    _evas_cache_image_entry_preload_remove(im, target);
-
-   pthread_mutex_lock(&mutex_pending);
-   if (!im->flags.pending && im->flags.in_pipe)
-     {
-	im->flags.pending = 1;
-	pending = eina_list_append(pending, im);
-     }
-   pthread_mutex_unlock(&mutex_pending);
-
 #else
-   (void) im;
+   (void)im;
 #endif
 }
 
@@ -1351,20 +1477,19 @@ evas_cache_image_flush(Evas_Cache_Image *cache)
 {
    assert(cache);
 
-   if (cache->limit == -1)
-     return -1;
+   if (cache->limit == -1) return -1;
 
    while ((cache->lru) && (cache->limit < cache->usage))
      {
-        Image_Entry     *im;
+        Image_Entry *im;
 
-        im = (Image_Entry *) cache->lru->last;
+        im = (Image_Entry *)cache->lru->last;
         _evas_cache_image_entry_delete(cache, im);
      }
 
    while ((cache->lru_nodata) && (cache->limit < cache->usage))
      {
-        Image_Entry     *im;
+        Image_Entry *im;
 
         im = (Image_Entry *) cache->lru_nodata->last;
         _evas_cache_image_remove_lru_nodata(cache, im);
@@ -1380,14 +1505,19 @@ evas_cache_image_flush(Evas_Cache_Image *cache)
 EAPI Image_Entry *
 evas_cache_image_empty(Evas_Cache_Image *cache)
 {
-   Image_Entry  *new;
+   Image_Entry *im;
 
-   new = _evas_cache_image_entry_new(cache, NULL, 0, NULL, NULL, NULL, NULL);
-   if (!new) return NULL;
+   im = _evas_cache_image_entry_new(cache, NULL, 0, NULL, NULL, NULL, NULL);
+   if (!im) return NULL;
 
-   new->references = 1;
-
-   return new;
+#ifdef EVAS_FRAME_QUEUING
+   LKL(im->lock_references);
+#endif
+   im->references = 1;
+#ifdef EVAS_FRAME_QUEUING
+   LKU(im->lock_references);
+#endif
+   return im;
 }
 
 EAPI void
@@ -1400,7 +1530,7 @@ evas_cache_image_colorspace(Image_Entry *im, int cspace)
 
    cache = im->cache;
 
-   if (im->space == cspace) return ;
+   if (im->space == cspace) return;
 
    im->space = cspace;
    cache->func.color_space(im, cspace);
@@ -1447,3 +1577,11 @@ evas_cache_image_pixels(Image_Entry *im)
 
    return cache->func.surface_pixels(im);
 }
+
+EAPI void
+evas_cache_image_wakeup(void)
+{
+#ifdef BUILD_ASYNC_PRELOAD
+   pthread_cond_broadcast(&cond_wakeup);
+#endif
+}
