@@ -72,16 +72,10 @@ static const Evas_Object_Func object_func =
 /* the actual api call to add a rect */
 /* it has no other api calls as all properties are standard */
 
-/**
- * @defgroup Evas_Line_Group Line Object Functions
- *
- * Functions used to deal with evas line objects.
- */
 
 /**
  * @addtogroup Evas_Line_Group
  * @{
- * @ingroup Evas_Object_Specific
  */
 
 /**
@@ -97,7 +91,7 @@ evas_object_line_add(Evas *e)
    MAGIC_CHECK(e, Evas, MAGIC_EVAS);
    return NULL;
    MAGIC_CHECK_END();
-   obj = evas_object_new();
+   obj = evas_object_new(e);
    evas_object_line_init(obj);
    evas_object_inject(obj, e);
    return obj;
