@@ -80,16 +80,9 @@ static const Evas_Object_Func object_func =
 /* the actual api call to add a rect */
 /* it has no other api calls as all properties are standard */
 
-/**
- * @defgroup Evas_Object_Text Text Object Functions
- *
- * Functions that operate on evas text objects.
- */
-
 /**
  * @addtogroup Evas_Object_Text
  * @{
- * @ingroup Evas_Object_Specific
  */
 
 /**
@@ -111,7 +104,7 @@ evas_object_text_add(Evas *e)
    MAGIC_CHECK(e, Evas, MAGIC_EVAS);
    return NULL;
    MAGIC_CHECK_END();
-   obj = evas_object_new();
+   obj = evas_object_new(e);
    evas_object_text_init(obj);
    evas_object_inject(obj, e);
    return obj;
@@ -202,6 +195,12 @@ evas_object_text_font_set(Evas_Object *obj, const char *font, Evas_Font_Size siz
 					      obj->layer->evas->pointer.x,
 					      obj->layer->evas->pointer.y, 1, 1);
      }
+
+#ifdef EVAS_FRAME_QUEUING
+   if (o->engine_data)
+      evas_common_pipe_op_text_flush(o->engine_data);
+#endif
+
    /* DO IT */
    if (o->engine_data)
      {
@@ -783,6 +782,7 @@ evas_object_text_shadow_color_set(Evas_Object *obj, int r, int g, int b, int a)
    o->cur.shadow.g = g;
    o->cur.shadow.b = b;
    o->cur.shadow.a = a;
+   o->changed = 1;
    evas_object_change(obj);
 }
 
@@ -835,6 +835,7 @@ evas_object_text_glow_color_set(Evas_Object *obj, int r, int g, int b, int a)
    o->cur.glow.g = g;
    o->cur.glow.b = b;
    o->cur.glow.a = a;
+   o->changed = 1;
    evas_object_change(obj);
 }
 
@@ -887,6 +888,7 @@ evas_object_text_glow2_color_set(Evas_Object *obj, int r, int g, int b, int a)
    o->cur.glow2.g = g;
    o->cur.glow2.b = b;
    o->cur.glow2.a = a;
+   o->changed = 1;
    evas_object_change(obj);
 }
 
@@ -939,6 +941,7 @@ evas_object_text_outline_color_set(Evas_Object *obj, int r, int g, int b, int a)
    o->cur.outline.g = g;
    o->cur.outline.b = b;
    o->cur.outline.a = a;
+   o->changed = 1;
    evas_object_change(obj);
 }
 
@@ -1002,11 +1005,6 @@ evas_object_text_style_pad_get(const Evas_Object *obj, int *l, int *r, int *t, i
  * @}
  */
 
-/**
- * @defgroup Evas_Font_Path_Group Evas Font Path Functions
- *
- * Functions that edit the paths being used to load fonts.
- */
 
 /**
  * Removes all font paths loaded into memory for the given evas.
@@ -1092,6 +1090,14 @@ evas_font_object_rehint(Evas_Object *obj)
      }
 }
 
+/**
+ * Changes the font hinting for the given evas.
+ *
+ * @param e The given evas.
+ * @param hinting The hinting to use, one of #EVAS_FONT_HINTING_NONE,
+ *        #EVAS_FONT_HINTING_AUTO, #EVAS_FONT_HINTING_BYTECODE.
+ * @ingroup Evas_Font_Group
+ */
 EAPI void
 evas_font_hinting_set(Evas *e, Evas_Font_Hinting_Flags hinting)
 {
@@ -1102,6 +1108,7 @@ evas_font_hinting_set(Evas *e, Evas_Font_Hinting_Flags hinting)
    MAGIC_CHECK_END();
    if (e->hinting == hinting) return;
    e->hinting = hinting;
+
    EINA_INLIST_FOREACH(e->layers, lay)
      {
 	Evas_Object *obj;
@@ -1111,6 +1118,14 @@ evas_font_hinting_set(Evas *e, Evas_Font_Hinting_Flags hinting)
      }
 }
 
+/**
+ * Retrieves the font hinting used by the given evas.
+ *
+ * @param e The given evas to query.
+ * @return The hinting in use, one of #EVAS_FONT_HINTING_NONE,
+ *         #EVAS_FONT_HINTING_AUTO, #EVAS_FONT_HINTING_BYTECODE.
+ * @ingroup Evas_Font_Group
+ */
 EAPI Evas_Font_Hinting_Flags
 evas_font_hinting_get(const Evas *e)
 {
@@ -1120,6 +1135,15 @@ evas_font_hinting_get(const Evas *e)
    return e->hinting;
 }
 
+/**
+ * Checks if the font hinting is supported by the given evas.
+ *
+ * @param e The given evas to query.
+ * @param hinting The hinting to use, one of #EVAS_FONT_HINTING_NONE,
+ *        #EVAS_FONT_HINTING_AUTO, #EVAS_FONT_HINTING_BYTECODE.
+ * @return @c EINA_TRUE if it is supported, @c EINA_FALSE otherwise.
+ * @ingroup Evas_Font_Group
+ */
 EAPI Eina_Bool
 evas_font_hinting_can_hint(const Evas *e, Evas_Font_Hinting_Flags hinting)
 {
@@ -1132,15 +1156,11 @@ evas_font_hinting_can_hint(const Evas *e, Evas_Font_Hinting_Flags hinting)
    return EINA_FALSE;
 }
 
-
-
-
-
 /**
- * To be documented.
- *
- * FIXME: To be fixed.
+ * Force the given evas and associated engine to flush its font cache.
  *
+ * @param e The given evas to flush font cache.
+ * @ingroup Evas_Font_Group
  */
 EAPI void
 evas_font_cache_flush(Evas *e)
@@ -1153,10 +1173,12 @@ evas_font_cache_flush(Evas *e)
 }
 
 /**
- * To be documented.
+ * Changes the size of font cache of the given evas.
  *
- * FIXME: To be fixed.
+ * @param e The given evas to flush font cache.
+ * @param size The size, in bytes.
  *
+ * @ingroup Evas_Font_Group
  */
 EAPI void
 evas_font_cache_set(Evas *e, int size)
@@ -1170,10 +1192,12 @@ evas_font_cache_set(Evas *e, int size)
 }
 
 /**
- * To be documented.
+ * Changes the size of font cache of the given evas.
  *
- * FIXME: To be fixed.
+ * @param e The given evas to flush font cache.
+ * @return The size, in bytes.
  *
+ * @ingroup Evas_Font_Group
  */
 EAPI int
 evas_font_cache_get(const Evas *e)
@@ -1186,10 +1210,18 @@ evas_font_cache_get(const Evas *e)
 }
 
 /**
- * To be documented.
+ * List of available font descriptions known or found by this evas.
  *
- * FIXME: To be fixed.
+ * The list depends on Evas compile time configuration, such as
+ * fontconfig support, and the paths provided at runtime as explained
+ * in @ref Evas_Font_Path_Group.
  *
+ * @param e The evas instance to query.
+ * @return a newly allocated list of strings. Do not change the
+ *         strings.  Be sure to call evas_font_available_list_free()
+ *         after you're done.
+ *
+ * @ingroup Evas_Font_Group
  */
 EAPI Eina_List *
 evas_font_available_list(const Evas *e)
@@ -1202,10 +1234,12 @@ evas_font_available_list(const Evas *e)
 }
 
 /**
- * To be documented.
+ * Free list of font descriptions returned by evas_font_dir_available_list().
  *
- * FIXME: To be fixed.
+ * @param e The evas instance that returned such list.
+ * @param available the list returned by evas_font_dir_available_list().
  *
+ * @ingroup Evas_Font_Group
  */
 EAPI void
 evas_font_available_list_free(Evas *e, Eina_List *available)
@@ -1222,6 +1256,7 @@ evas_font_available_list_free(Evas *e, Eina_List *available)
  *
  * FIXME: To be fixed.
  *
+ * @ingroup Evas_Utils
  */
 EAPI int
 evas_string_char_next_get(const char *str, int pos, int *decoded)
@@ -1242,6 +1277,7 @@ evas_string_char_next_get(const char *str, int pos, int *decoded)
  *
  * FIXME: To be fixed.
  *
+ * @ingroup Evas_Utils
  */
 EAPI int
 evas_string_char_prev_get(const char *str, int pos, int *decoded)
@@ -1261,12 +1297,13 @@ evas_string_char_prev_get(const char *str, int pos, int *decoded)
  * Get the length in characters of the string.
  * @param  str The string to get the length of.
  * @return The length in characters (not bytes)
+ * @ingroup Evas_Utils
  */
 EAPI int
 evas_string_char_len_get(const char *str)
 {
 	if (!str) return 0;
-	return evas_common_font_utf8_get_len(str);
+	return evas_common_font_utf8_get_len((const unsigned char *) str);
 }
 
 /**
@@ -1804,6 +1841,9 @@ _evas_object_text_rehint(Evas_Object *obj)
 
    o = (Evas_Object_Text *)(obj->object_data);
    if (!o->engine_data) return;
+#ifdef EVAS_FRAME_QUEUING
+   evas_common_pipe_op_text_flush(o->engine_data);
+#endif
    evas_font_load_hinting_set(obj->layer->evas, o->engine_data,
 			      obj->layer->evas->hinting);
    was = evas_object_is_in_output_rect(obj,
