@@ -4,17 +4,14 @@
 
 #include "evas_common.h"
 #include "evas_convert_main.h"
+#include "evas_private.h"
 
 EAPI Cutout_Rects*
 evas_common_draw_context_cutouts_new(void)
 {
    Cutout_Rects *rects;
 
-   rects = malloc(sizeof(Cutout_Rects));
-   rects->rects = NULL;
-   rects->active = 0;
-   rects->max = 0;
-
+   rects = calloc(1, sizeof(Cutout_Rects));
    return rects;
 }
 
@@ -24,27 +21,6 @@ evas_common_draw_context_cutouts_free(Cutout_Rects* rects)
    rects->active = 0;
 }
 
-static Cutout_Rect*
-evas_common_draw_context_cutouts_add(Cutout_Rects* rects,
-                                     int x, int y, int w, int h)
-{
-   Cutout_Rect* rect;
-
-   rects->active++;
-   if (rects->max < rects->active)
-     {
-	rects->max += 1024;
-	rects->rects = realloc(rects->rects, sizeof(Cutout_Rect) * rects->max);
-     }
-   rect = rects->rects + rects->active - 1;
-   rect->x = x;
-   rect->y = y;
-   rect->w = w;
-   rect->h = h;
-
-   return rect;
-}
-
 EAPI void
 evas_common_draw_context_cutouts_del(Cutout_Rects* rects,
                                      int index)
