@@ -7,6 +7,9 @@
 FT_Library      evas_ft_lib = 0;
 static int      initialised = 0;
 
+LK(lock_font_draw); // for freetype2 API calls
+LK(lock_fribidi); // for fribidi API calls
+
 EAPI void
 evas_common_font_init(void)
 {
@@ -15,12 +18,13 @@ evas_common_font_init(void)
    initialised++;
    if (initialised != 1) return;
    error = FT_Init_FreeType(&evas_ft_lib);
-   if (error)
-     {
-	initialised--;
-	return;
-     }
+   if (error) return;
    evas_common_font_load_init();
+#ifdef EVAS_FRAME_QUEUING
+   evas_common_font_draw_init();
+#endif
+   LKI(lock_font_draw);
+   LKI(lock_fribidi);
 }
 
 EAPI void
@@ -28,17 +32,30 @@ evas_common_font_shutdown(void)
 {
    int error;
 
+   if (initialised < 1) return;
    initialised--;
    if (initialised != 0) return;
 
+   LKD(lock_font_draw);
+   LKD(lock_fribidi);
+   
    evas_common_font_load_shutdown();
    evas_common_font_cache_set(0);
    evas_common_font_flush();
 
    error = FT_Done_FreeType(evas_ft_lib);
+#ifdef EVAS_FRAME_QUEUING
+   evas_common_font_draw_finish();
+#endif
    evas_ft_lib = 0;
 }
 
+EAPI void
+evas_common_font_font_all_unload(void)
+{
+   evas_common_font_all_clear();
+}
+
 EAPI int
 evas_common_font_ascent_get(RGBA_Font *fn)
 {
@@ -46,14 +63,46 @@ evas_common_font_ascent_get(RGBA_Font *fn)
    int ret;
    RGBA_Font_Int *fi;
 
-   evas_common_font_size_use(fn);
+//   evas_common_font_size_use(fn);
+#if 0
+     {
+        Eina_List *l;
+        
+        EINA_LIST_FOREACH(fn->fonts, l, fi)
+          {
+             if (!fi->src->ft.face) continue;
+             if (fi->src->current_size != fi->size)
+               {
+                  FT_Activate_Size(fi->ft.size);
+                  fi->src->current_size = fi->size;
+               }
+             val = (int)fi->src->ft.face->size->metrics.ascender;
+             if (fi->src->ft.face->units_per_EM == 0)
+               return val;
+             dv = (fi->src->ft.orig_upem * 2048) / fi->src->ft.face->units_per_EM;
+             ret = (val * fi->src->ft.face->size->metrics.y_scale) / (dv * dv);
+             printf(" ==== %p: %i\n", fi, ret);
+          }
+     }
+#endif
    fi = fn->fonts->data;
+   if (fi->src->current_size != fi->size)
+     {
+        FT_Activate_Size(fi->ft.size);
+        fi->src->current_size = fi->size;
+     }
+   if (!FT_IS_SCALABLE(fi->src->ft.face))
+     {
+        printf("NOT SCALABLE!\n");
+     }
    val = (int)fi->src->ft.face->size->metrics.ascender;
-   if (fi->src->ft.face->units_per_EM == 0)
-     return val;
-   dv = (fi->src->ft.orig_upem * 2048) / fi->src->ft.face->units_per_EM;
-   ret = (val * fi->src->ft.face->size->metrics.y_scale) / (dv * dv);
-   return ret;
+   return val >> 6;
+//   printf("%i | %i\n", val, val >> 6);
+//   if (fi->src->ft.face->units_per_EM == 0)
+//     return val;
+//   dv = (fi->src->ft.orig_upem * 2048) / fi->src->ft.face->units_per_EM;
+//   ret = (val * fi->src->ft.face->size->metrics.y_scale) / (dv * dv);
+//   return ret;
 }
 
 EAPI int
@@ -63,14 +112,20 @@ evas_common_font_descent_get(RGBA_Font *fn)
    int ret;
    RGBA_Font_Int *fi;
 
-   evas_common_font_size_use(fn);
+//   evas_common_font_size_use(fn);
    fi = fn->fonts->data;
+   if (fi->src->current_size != fi->size)
+     {
+        FT_Activate_Size(fi->ft.size);
+        fi->src->current_size = fi->size;
+     }
    val = -(int)fi->src->ft.face->size->metrics.descender;
-   if (fi->src->ft.face->units_per_EM == 0)
-     return val;
-   dv = (fi->src->ft.orig_upem * 2048) / fi->src->ft.face->units_per_EM;
-   ret = (val * fi->src->ft.face->size->metrics.y_scale) / (dv * dv);
-   return ret;
+   return val >> 6;
+//   if (fi->src->ft.face->units_per_EM == 0)
+//     return val;
+//   dv = (fi->src->ft.orig_upem * 2048) / fi->src->ft.face->units_per_EM;
+//   ret = (val * fi->src->ft.face->size->metrics.y_scale) / (dv * dv);
+//   return ret;
 }
 
 EAPI int
@@ -80,8 +135,13 @@ evas_common_font_max_ascent_get(RGBA_Font *fn)
    int ret;
    RGBA_Font_Int *fi;
 
-   evas_common_font_size_use(fn);
+//   evas_common_font_size_use(fn);
    fi = fn->fonts->data;
+   if (fi->src->current_size != fi->size)
+     {
+        FT_Activate_Size(fi->ft.size);
+        fi->src->current_size = fi->size;
+     }
    val = (int)fi->src->ft.face->bbox.yMax;
    if (fi->src->ft.face->units_per_EM == 0)
      return val;
@@ -97,8 +157,13 @@ evas_common_font_max_descent_get(RGBA_Font *fn)
    int ret;
    RGBA_Font_Int *fi;
 
-   evas_common_font_size_use(fn);
+//   evas_common_font_size_use(fn);
    fi = fn->fonts->data;
+   if (fi->src->current_size != fi->size)
+     {
+        FT_Activate_Size(fi->ft.size);
+        fi->src->current_size = fi->size;
+     }
    val = -(int)fi->src->ft.face->bbox.yMin;
    if (fi->src->ft.face->units_per_EM == 0)
      return val;
@@ -114,14 +179,20 @@ evas_common_font_get_line_advance(RGBA_Font *fn)
    int ret;
    RGBA_Font_Int *fi;
 
-   evas_common_font_size_use(fn);
+//   evas_common_font_size_use(fn);
    fi = fn->fonts->data;
+   if (fi->src->current_size != fi->size)
+     {
+        FT_Activate_Size(fi->ft.size);
+        fi->src->current_size = fi->size;
+     }
    val = (int)fi->src->ft.face->size->metrics.height;
    if (fi->src->ft.face->units_per_EM == 0)
      return val;
-   dv = (fi->src->ft.orig_upem * 2048) / fi->src->ft.face->units_per_EM;
-   ret = (val * fi->src->ft.face->size->metrics.y_scale) / (dv * dv);
-   return ret;
+   return val >> 6;
+//   dv = (fi->src->ft.orig_upem * 2048) / fi->src->ft.face->units_per_EM;
+//   ret = (val * fi->src->ft.face->size->metrics.y_scale) / (dv * dv);
+//   return ret;
 }
 
 EAPI int
