@@ -1,6 +1,6 @@
 #include "evas_common.h"
-
 #include "evas_intl_utils.h" /*defines INTERNATIONAL_SUPPORT if possible */
+#include "evas_font_private.h" /* for Frame-Queuing support */
 
 EAPI int
 evas_common_font_query_kerning(RGBA_Font_Int* fi,
@@ -12,13 +12,10 @@ evas_common_font_query_kerning(RGBA_Font_Int* fi,
    int key[2];
    int error = 1;
 
+//   return 0;
    key[0] = prev;
    key[1] = index;
 
-#ifdef HAVE_PTHREAD
-   pthread_mutex_lock(&fi->ft_mutex);
-#endif
-
    result = eina_hash_find(fi->kerning, key);
    if (result)
      {
@@ -30,12 +27,14 @@ evas_common_font_query_kerning(RGBA_Font_Int* fi,
     * values to kern by - given same font, same size and same
     * prev_index and index. auto/bytecode or none hinting doesnt
     * matter */
+   FTLOCK();
    if (FT_Get_Kerning(fi->src->ft.face,
 		      key[0], key[1],
 		      ft_kerning_default, &delta) == 0)
      {
 	int *push;
 
+        FTUNLOCK();
 	*kerning = delta.x >> 6;
 
 	push = malloc(sizeof (int) * 3);
@@ -50,12 +49,10 @@ evas_common_font_query_kerning(RGBA_Font_Int* fi,
 	goto on_correct;
      }
 
+        FTUNLOCK();
    error = 0;
 
  on_correct:
-#ifdef HAVE_PTHREAD
-   pthread_mutex_unlock(&fi->ft_mutex);
-#endif
    return error;
 }
 
@@ -78,7 +75,7 @@ evas_common_font_query_size(RGBA_Font *fn, const char *text, int *w, int *h)
 
    pen_x = 0;
    pen_y = 0;
-   evas_common_font_size_use(fn);
+//   evas_common_font_size_use(fn);
    use_kerning = FT_HAS_KERNING(fi->src->ft.face);
    prev_index = 0;
    for (chr = 0; text[chr];)
@@ -91,7 +88,13 @@ evas_common_font_query_size(RGBA_Font *fn, const char *text, int *w, int *h)
 	gl = evas_common_font_utf8_get_next((unsigned char *)text, &chr);
 	if (gl == 0) break;
 	index = evas_common_font_glyph_search(fn, &fi, gl);
-        /* hmmm kerning means i can't sanely do my own cached metric tables! */
+	LKL(fi->ft_mutex);
+        if (fi->src->current_size != fi->size)
+          {
+             FT_Activate_Size(fi->ft.size);
+             fi->src->current_size = fi->size;
+          }
+      /* hmmm kerning means i can't sanely do my own cached metric tables! */
 	/* grrr - this means font face sharing is kinda... not an option if */
 	/* you want performance */
 	kern = 0;
@@ -102,6 +105,7 @@ evas_common_font_query_size(RGBA_Font *fn, const char *text, int *w, int *h)
 
 	pface = fi->src->ft.face;
 	fg = evas_common_font_int_cache_glyph_get(fi, index);
+	LKU(fi->ft_mutex);
 	if (!fg) continue;
 
 	if (kern < 0) kern = 0;
@@ -144,8 +148,13 @@ evas_common_font_query_inset(RGBA_Font *fn, const char *text)
    if (!text[0]) return 0;
    gl = evas_common_font_utf8_get_next((unsigned char *)text, &chr);
    if (gl == 0) return 0;
-   evas_common_font_size_use(fn);
+//   evas_common_font_size_use(fn);
    index = evas_common_font_glyph_search(fn, &fi, gl);
+   if (fi->src->current_size != fi->size)
+     {
+        FT_Activate_Size(fi->ft.size);
+        fi->src->current_size = fi->size;
+     }
    fg = evas_common_font_int_cache_glyph_get(fi, index);
    if (!fg) return 0;
 /*
@@ -182,8 +191,10 @@ evas_common_font_query_advance(RGBA_Font *fn, const char *text, int *h_adv, int
    start_x = 0;
    pen_x = 0;
    pen_y = 0;
-   evas_common_font_size_use(fn);
+//   evas_common_font_size_use(fn);
+   FTLOCK();
    use_kerning = FT_HAS_KERNING(fi->src->ft.face);
+   FTUNLOCK();
    prev_index = 0;
    for (chr = 0; text[chr];)
      {
@@ -195,7 +206,13 @@ evas_common_font_query_advance(RGBA_Font *fn, const char *text, int *h_adv, int
 	gl = evas_common_font_utf8_get_next((unsigned char *)text, &chr);
 	if (gl == 0) break;
 	index = evas_common_font_glyph_search(fn, &fi, gl);
-        /* hmmm kerning means i can't sanely do my own cached metric tables! */
+	LKL(fi->ft_mutex);
+        if (fi->src->current_size != fi->size)
+          {
+             FT_Activate_Size(fi->ft.size);
+             fi->src->current_size = fi->size;
+          }
+      /* hmmm kerning means i can't sanely do my own cached metric tables! */
 	/* grrr - this means font face sharing is kinda... not an option if */
 	/* you want performance */
 	if ((use_kerning) && (prev_index) && (index) &&
@@ -205,6 +222,7 @@ evas_common_font_query_advance(RGBA_Font *fn, const char *text, int *h_adv, int
 
 	pface = fi->src->ft.face;
 	fg = evas_common_font_int_cache_glyph_get(fi, index);
+	LKU(fi->ft_mutex);
 	if (!fg) continue;
 
         chr_x = (pen_x + (fg->glyph_out->left));
@@ -248,7 +266,12 @@ evas_common_font_query_char_coords(RGBA_Font *fn, const char *in_text, int pos,
 
    pen_x = 0;
    pen_y = 0;
-   evas_common_font_size_use(fn);
+//   evas_common_font_size_use(fn);
+   if (fi->src->current_size != fi->size)
+     {
+        FT_Activate_Size(fi->ft.size);
+        fi->src->current_size = fi->size;
+     }
    use_kerning = FT_HAS_KERNING(fi->src->ft.face);
    prev_index = 0;
    prev_chr_end = 0;
@@ -285,6 +308,8 @@ evas_common_font_query_char_coords(RGBA_Font *fn, const char *in_text, int pos,
 	gl = evas_common_font_utf8_get_next((unsigned char *)text, &chr);
 	if (gl == 0) break;
 	index = evas_common_font_glyph_search(fn, &fi, gl);
+	LKL(fi->ft_mutex);
+	// FIXME: Why no FT_Activate_Size here ?
 	kern = 0;
         /* hmmm kerning means i can't sanely do my own cached metric tables! */
 	/* grrr - this means font face sharing is kinda... not an option if */
@@ -311,6 +336,7 @@ evas_common_font_query_char_coords(RGBA_Font *fn, const char *in_text, int pos,
 
 	pface = fi->src->ft.face;
 	fg = evas_common_font_int_cache_glyph_get(fi, index);
+	LKU(fi->ft_mutex);
 	if (!fg) continue;
 
 	if (kern < 0) kern = 0;
@@ -383,7 +409,12 @@ evas_common_font_query_text_at_pos(RGBA_Font *fn, const char *in_text, int x, in
 
    pen_x = 0;
    pen_y = 0;
-   evas_common_font_size_use(fn);
+//   evas_common_font_size_use(fn);
+   if (fi->src->current_size != fi->size)
+     {
+        FT_Activate_Size(fi->ft.size);
+        fi->src->current_size = fi->size;
+     }
    use_kerning = FT_HAS_KERNING(fi->src->ft.face);
    prev_index = 0;
    prev_chr_end = 0;
@@ -402,6 +433,8 @@ evas_common_font_query_text_at_pos(RGBA_Font *fn, const char *in_text, int x, in
 	gl = evas_common_font_utf8_get_next((unsigned char *)text, &chr);
 	if (gl == 0) break;
 	index = evas_common_font_glyph_search(fn, &fi, gl);
+	LKL(fi->ft_mutex);
+	// FIXME: Why not FT_Activate_Size here ?
 	kern = 0;
         /* hmmm kerning means i can't sanely do my own cached metric tables! */
 	/* grrr - this means font face sharing is kinda... not an option if */
@@ -428,6 +461,7 @@ evas_common_font_query_text_at_pos(RGBA_Font *fn, const char *in_text, int x, in
 
 	pface = fi->src->ft.face;
 	fg = evas_common_font_int_cache_glyph_get(fi, index);
+	LKU(fi->ft_mutex);
 	if (!fg) continue;
 
 	if (kern < 0) kern = 0;
@@ -504,7 +538,7 @@ evas_common_font_query_last_up_to_pos(RGBA_Font *fn, const char *text, int x, in
 
    pen_x = 0;
    pen_y = 0;
-   evas_common_font_size_use(fn);
+//   evas_common_font_size_use(fn);
    use_kerning = FT_HAS_KERNING(fi->src->ft.face);
    prev_index = 0;
    prev_chr_end = 0;
@@ -522,6 +556,12 @@ evas_common_font_query_last_up_to_pos(RGBA_Font *fn, const char *text, int x, in
 	gl = evas_common_font_utf8_get_next((unsigned char *)text, &chr);
 	if (gl == 0) break;
 	index = evas_common_font_glyph_search(fn, &fi, gl);
+	LKL(fi->ft_mutex);
+        if (fi->src->current_size != fi->size)
+          {
+             FT_Activate_Size(fi->ft.size);
+             fi->src->current_size = fi->size;
+          }
 	kern = 0;
         /* hmmm kerning means i can't sanely do my own cached metric tables! */
 	/* grrr - this means font face sharing is kinda... not an option if */
@@ -533,6 +573,7 @@ evas_common_font_query_last_up_to_pos(RGBA_Font *fn, const char *text, int x, in
 
 	pface = fi->src->ft.face;
 	fg = evas_common_font_int_cache_glyph_get(fi, index);
+	LKU(fi->ft_mutex);
 	if (!fg) continue;
 
 	if (kern < 0) kern = 0;
