@@ -44,12 +44,12 @@ _evas_common_gradient_stops_free(RGBA_Gradient *gr)
    if (!gr) return;
    if (gr->color.stops)
      {
-	Evas_Object_List *l;
+	Eina_Inlist *l;
 
 	while (gr->color.stops)
 	  {
 	   l = gr->color.stops;
-	   gr->color.stops = evas_object_list_remove(gr->color.stops, gr->color.stops);
+	   gr->color.stops = eina_inlist_remove(gr->color.stops, gr->color.stops);
 	   free(l);
 	  }
 	gr->color.stops = NULL;
@@ -57,12 +57,12 @@ _evas_common_gradient_stops_free(RGBA_Gradient *gr)
      }
    if (gr->alpha.stops)
      {
-	Evas_Object_List *l;
+	Eina_Inlist *l;
 
 	while (gr->alpha.stops)
 	  {
 	   l = gr->alpha.stops;
-	   gr->alpha.stops = evas_object_list_remove(gr->alpha.stops, gr->alpha.stops);
+	   gr->alpha.stops = eina_inlist_remove(gr->alpha.stops, gr->alpha.stops);
 	   free(l);
 	  }
 	gr->alpha.stops = NULL;
@@ -160,6 +160,15 @@ evas_common_gradient_free(RGBA_Gradient *gr)
    if (!gr) return;
    gr->references--;
    if (gr->references > 0) return;
+#ifdef EVAS_FRAME_QUEUING
+        LKL(gr->ref_fq_add);  LKL(gr->ref_fq_del);
+        if (gr->ref_fq[0] != gr->ref_fq[1])
+          {
+             LKU(gr->ref_fq_add); LKU(gr->ref_fq_del);
+             return;
+          }
+	LKU(gr->ref_fq_add); LKU(gr->ref_fq_del);
+#endif
    evas_common_gradient_clear(gr);
    if (gr->type.name) free(gr->type.name);
    if (gr->type.params) free(gr->type.params);
@@ -220,7 +229,7 @@ evas_common_gradient_color_stop_add(RGBA_Gradient *gr, int r, int g, int b, int
 
    if (!gr->color.stops)
      {
-	gr->color.stops = evas_object_list_append(gr->color.stops, gc);
+       gr->color.stops = eina_inlist_append(gr->color.stops, EINA_INLIST_GET(gc));
 	gr->color.nstops = 1;
 	gr->color.len = 1;
 	if (a < 255)
@@ -237,9 +246,9 @@ evas_common_gradient_color_stop_add(RGBA_Gradient *gr, int r, int g, int b, int
    gcm->b = (gc_last->b + b) / 2;
    gcm->a = (gc_last->a + a) / 2;
    gcm->dist = dist;
-   gr->color.stops = evas_object_list_append(gr->color.stops, gcm);
+   gr->color.stops = eina_inlist_append(gr->color.stops, EINA_INLIST_GET(gcm));
    gr->color.len += gc_last->dist;
-   gr->color.stops = evas_object_list_append(gr->color.stops, gc);
+   gr->color.stops = eina_inlist_append(gr->color.stops, EINA_INLIST_GET(gc));
    gr->color.len += dist;
    gr->color.nstops += 2;
    if (a < 255)
@@ -271,7 +280,7 @@ evas_common_gradient_alpha_stop_add(RGBA_Gradient *gr, int a, int dist)
 
    if (!gr->alpha.stops)
      {
-	gr->alpha.stops = evas_object_list_append(gr->alpha.stops, ga);
+        gr->alpha.stops = eina_inlist_append(gr->alpha.stops, EINA_INLIST_GET(ga));
 	gr->alpha.nstops = 1;
 	gr->alpha.len = 1;
 	if (a < 255)
@@ -285,9 +294,9 @@ evas_common_gradient_alpha_stop_add(RGBA_Gradient *gr, int a, int dist)
 	{ free(ga); free(gam); return; }
    gam->a = (ga_last->a + a) / 2;
    gam->dist = dist;
-   gr->alpha.stops = evas_object_list_append(gr->alpha.stops, gam);
+   gr->alpha.stops = eina_inlist_append(gr->alpha.stops, EINA_INLIST_GET(gam));
    gr->alpha.len += ga_last->dist;
-   gr->alpha.stops = evas_object_list_append(gr->alpha.stops, ga);
+   gr->alpha.stops = eina_inlist_append(gr->alpha.stops, EINA_INLIST_GET(ga));
    gr->alpha.len += dist;
    gr->alpha.nstops += 2;
    if (a < 255)
@@ -427,7 +436,7 @@ evas_common_gradient_draw(RGBA_Image *dst, RGBA_Draw_Context *dc,
 			  int x, int y, int w, int h, RGBA_Gradient *gr)
 {
    Gfx_Func_Gradient_Fill   gfunc;
-   RGBA_Gfx_Func            bfunc;
+   RGBA_Gfx_Func            bfunc = NULL;
    int             len;
    int             xin, yin, xoff, yoff;
    int             clx, cly, clw, clh;
@@ -617,7 +626,7 @@ evas_common_gradient_map_argb(RGBA_Draw_Context *dc, RGBA_Gradient *gr, int len)
 
    if (gr->color.stops)
      {
-	Evas_Object_List  *lc;
+	Eina_Inlist  *lc;
 	RGBA_Gradient_Color_Stop  *gc, *gc_next;
 	DATA32  *pmap, *map_end;
 	int   i, dii;
@@ -681,7 +690,7 @@ evas_common_gradient_map_argb(RGBA_Draw_Context *dc, RGBA_Gradient *gr, int len)
 
    if (gr->alpha.stops)
      {
-	Evas_Object_List  *lc;
+	Eina_Inlist  *lc;
 	RGBA_Gradient_Alpha_Stop  *ga, *ga_next;
 	DATA8  *pamap, *amap_end;
 	int   i, dii;
@@ -789,7 +798,7 @@ evas_common_gradient_map_ahsv(RGBA_Draw_Context *dc, RGBA_Gradient *gr, int len)
 
    if (gr->color.stops)
      {
-	Evas_Object_List  *lc;
+	Eina_Inlist  *lc;
 	RGBA_Gradient_Color_Stop  *gc, *gc_next;
 	DATA32  *pmap, *map_end;
 	int   i, dii;
@@ -846,7 +855,7 @@ evas_common_gradient_map_ahsv(RGBA_Draw_Context *dc, RGBA_Gradient *gr, int len)
 
    if (gr->alpha.stops)
      {
-	Evas_Object_List  *lc;
+	Eina_Inlist  *lc;
 	RGBA_Gradient_Alpha_Stop  *ga, *ga_next;
 	DATA8  *pamap, *amap_end;
 	int   i, dii;
