@@ -2,6 +2,14 @@
  * vim:ts=8:sw=3:sts=8:noexpandtab:cino=>5n-3f0^-2{2
  */
 
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
+
+#ifdef HAVE_EVIL
+# include <Evil.h>
+#endif
+
 #include <assert.h>
 
 #include "evas_common.h"
@@ -32,6 +40,9 @@ struct _Scaleitem
    int dst_w, dst_h;
    int flop;
    int size_adjust;
+#ifdef EVAS_FRAME_QUEUING
+   RWLK(lock);
+#endif
    Eina_Bool forced_unload : 1;
    Eina_Bool smooth : 1;
    Eina_Bool populate_me : 1;
@@ -82,7 +93,8 @@ evas_common_scalecache_shutdown(void)
 {
 #ifdef SCALECACHE
    init--;
-   LKD(cache_lock);
+   if (init ==0)
+      LKD(cache_lock);
 #endif
 }
 
@@ -115,6 +127,9 @@ evas_common_rgba_image_scalecache_dirty(Image_Entry *ie)
      {
         Scaleitem *sci;
         sci = im->cache.list->data;
+#ifdef EVAS_FRAME_QUEUING
+        WRLKL(sci->lock);
+#endif
         im->cache.list = eina_list_remove(im->cache.list, sci);
         if (sci->im)
           {
@@ -128,6 +143,10 @@ evas_common_rgba_image_scalecache_dirty(Image_Entry *ie)
              cache_list = eina_inlist_remove(cache_list, (Eina_Inlist *)sci);
              LKU(cache_lock);
           }
+#ifdef EVAS_FRAME_QUEUING
+         RWLKU(sci->lock);
+         RWLKD(sci->lock);
+#endif
         free(sci);
      }
    LKU(im->cache.lock);
@@ -222,6 +241,9 @@ _sci_find(RGBA_Image *im,
      {
         l = eina_list_last(im->cache.list);
         sci = l->data;
+#ifdef EVAS_FRAME_QUEUING
+        WRLKL(sci->lock);
+#endif
         im->cache.list = eina_list_remove_list(im->cache.list, l);
         if ((sci->usage == im->cache.newest_usage) ||
             (sci->usage_count == im->cache.newest_usage_count))
@@ -236,6 +258,9 @@ _sci_find(RGBA_Image *im,
 //             INF(" 1- %i", sci->dst_w * sci->dst_h * 4);
              cache_list = eina_inlist_remove(cache_list, (Eina_Inlist *)sci);
           }
+#ifdef EVAS_FRAME_QUEUING
+        RWLKU(sci->lock);
+#endif
         if (max_scale_items < 1) return NULL;
      }
    else
@@ -247,6 +272,9 @@ _sci_find(RGBA_Image *im,
         sci = malloc(sizeof(Scaleitem));
         memset(sci, 0, sizeof(Eina_Inlist));
         sci->parent_im = im;
+#ifdef EVAS_FRAME_QUEUING
+        RWLKI(sci->lock);
+#endif
      }
    sci->usage = 0;
    sci->usage_count = 0;
@@ -280,6 +308,9 @@ _cache_prune(Scaleitem *notsci, Eina_Bool copies_only)
              if (!sci) return;
           }
         if (sci == notsci) return;
+#ifdef EVAS_FRAME_QUEUING
+        WRLKL(sci->lock);
+#endif
         if (sci->im)
           {
              evas_common_rgba_image_free(&sci->im->cache_entry);
@@ -295,6 +326,10 @@ _cache_prune(Scaleitem *notsci, Eina_Bool copies_only)
              cache_list = eina_inlist_remove(cache_list, (Eina_Inlist *)sci);
              memset(sci, 0, sizeof(Eina_Inlist));
           }
+#ifdef EVAS_FRAME_QUEUING
+        RWLKU(sci->lock);
+#endif
+
 //        INF("FLUSH %i > %i", cache_size, max_cache_size);
       }
 }
@@ -459,10 +494,16 @@ evas_common_rgba_image_scalecache_do(Image_Entry *ie, RGBA_Image *dst,
    LKL(im->cache.lock);
    if ((src_region_w == dst_region_w) && (src_region_h == dst_region_h))
      {
-        if (im->cache_entry.space == EVAS_COLORSPACE_ARGB8888)
-          evas_cache_image_load_data(&im->cache_entry);
-        evas_common_image_colorspace_normalize(im);
+#ifdef EVAS_FRAME_QUEUING
+        if (!evas_common_frameq_enabled())
+#endif
+         {
+            if (im->cache_entry.space == EVAS_COLORSPACE_ARGB8888)
+               evas_cache_image_load_data(&im->cache_entry);
+            evas_common_image_colorspace_normalize(im);
+         }
 //        noscales++;
+        LKU(im->cache.lock);
         if (im->image.data)
           {
              if (smooth)
@@ -478,7 +519,6 @@ evas_common_rgba_image_scalecache_do(Image_Entry *ie, RGBA_Image *dst,
                                                             dst_region_x, dst_region_y, 
                                                             dst_region_w, dst_region_h);
           }
-        LKU(im->cache.lock);
         return;
      }
    LKL(cache_lock);
@@ -488,9 +528,14 @@ evas_common_rgba_image_scalecache_do(Image_Entry *ie, RGBA_Image *dst,
    LKU(cache_lock);
    if (!sci)
      {
-        if (im->cache_entry.space == EVAS_COLORSPACE_ARGB8888)
-          evas_cache_image_load_data(&im->cache_entry);
-        evas_common_image_colorspace_normalize(im);
+#ifdef EVAS_FRAME_QUEUING
+        if (!evas_common_frameq_enabled())
+#endif
+          {
+             if (im->cache_entry.space == EVAS_COLORSPACE_ARGB8888)
+                evas_cache_image_load_data(&im->cache_entry);
+             evas_common_image_colorspace_normalize(im);
+          }
 //        misses++;
         LKU(im->cache.lock);
         if (im->image.data)
@@ -638,25 +683,42 @@ evas_common_rgba_image_scalecache_do(Image_Entry *ie, RGBA_Image *dst,
      {
         if (!didpop)
           {
+	     LKL(cache_lock);
              cache_list = eina_inlist_remove(cache_list, (Eina_Inlist *)sci);
              cache_list = eina_inlist_append(cache_list, (Eina_Inlist *)sci);
+	     LKU(cache_lock);
           }
         else
           {
              if (sci->flop > 0) sci->flop -= FLOP_DEL;
           }
 //        INF("use cached!");
+#ifdef EVAS_FRAME_QUEUING
+        RDLKL(sci->lock);
+#endif
+        LKU(im->cache.lock);
         evas_common_scale_rgba_in_to_out_clip_sample
           (sci->im, dst, dc,
            0, 0,
            dst_region_w, dst_region_h,
            dst_region_x, dst_region_y, 
            dst_region_w, dst_region_h);
+#ifdef EVAS_FRAME_QUEUING
+        RWLKU(sci->lock);
+#endif
 //        hits++;
 //        INF("check %p %i < %i", 
 //               im,
 //               (int)im->cache.orig_usage, 
 //               (int)im->cache.newest_usage);
+#ifndef EVAS_FRAME_QUEUING
+        /* while framequeuing is applied,
+         * original image data is loaded by the main thread 
+         * just before enqueuing the rendering op into the pipe.
+         * so unloading the original image data here 
+         * causes only speed-down side-effect and no memory usage gain;
+         * it will be loaded again for the very next rendering for this image.
+         */
         if ((dounload) || 
             ((im->cache_entry.flags.loaded) && 
              ((!im->cs.no_free) 
@@ -672,13 +734,18 @@ evas_common_rgba_image_scalecache_do(Image_Entry *ie, RGBA_Image *dst,
                   evas_common_rgba_image_unload(&im->cache_entry);
                }
           }
-        LKU(im->cache.lock);
+#endif
      }
    else
      {
-        if (im->cache_entry.space == EVAS_COLORSPACE_ARGB8888)
-          evas_cache_image_load_data(&im->cache_entry);
-        evas_common_image_colorspace_normalize(im);
+#ifdef EVAS_FRAME_QUEUING
+        if (!evas_common_frameq_enabled())
+#endif
+          {
+             if (im->cache_entry.space == EVAS_COLORSPACE_ARGB8888)
+                evas_cache_image_load_data(&im->cache_entry);
+             evas_common_image_colorspace_normalize(im);
+          }
 //        misses++;
         LKU(im->cache.lock);
         if (im->image.data)
@@ -699,9 +766,14 @@ evas_common_rgba_image_scalecache_do(Image_Entry *ie, RGBA_Image *dst,
      }
 #else   
    RGBA_Image *im = (RGBA_Image *)ie;
-   if (im->cache_entry.space == EVAS_COLORSPACE_ARGB8888)
-     evas_cache_image_load_data(&im->cache_entry);
-   evas_common_image_colorspace_normalize(im);
+#ifdef EVAS_FRAME_QUEUING
+   if (!evas_common_frameq_enabled())
+#endif
+     {
+        if (im->cache_entry.space == EVAS_COLORSPACE_ARGB8888)
+           evas_cache_image_load_data(&im->cache_entry);
+        evas_common_image_colorspace_normalize(im);
+     }
    if (im->image.data)
      {
         if (smooth)
