@@ -377,10 +377,10 @@ fb_getmode(void)
       bpp = mode->fb_var.bits_per_pixel / 8;
       break;
      default:
-      fprintf(stderr, "Cannot handle framebuffer of depth %i\n",
-	      mode->fb_var.bits_per_pixel);
+      ERR("Cannot handle framebuffer of depth %i",
+	     mode->fb_var.bits_per_pixel);
       fb_cleanup();
-       free(mode);
+      free(mode);
       return NULL;
   }
   mode->depth = depth;
@@ -418,7 +418,7 @@ fb_setvt(int vtno)
   chown(vtname, getuid(), getgid());
   if (access(vtname,R_OK | W_OK) == -1)
     {
-      fprintf(stderr,"access %s: %s\n",vtname,strerror(errno));
+      CRIT("Access %s: %s",vtname,strerror(errno));
       exit(1);
     }
   open(vtname,O_RDWR);
@@ -463,7 +463,7 @@ fb_init(int vt, int device)
      }
    if (fb == -1)
      {
-	fprintf(stderr,"open %s: %s\n", dev, strerror(errno));
+	CRIT("open %s: %s", dev, strerror(errno));
 	fb_cleanup();
 	exit(1);
      }
@@ -495,7 +495,7 @@ fb_init(int vt, int device)
       tty = 0;
    else if ((tty = open("/dev/tty",O_RDWR)) == -1)
      {
-	fprintf(stderr,"open %s: %s\n", "/dev/tty", strerror(errno));
+	CITICAL("open %s: %s", "/dev/tty", strerror(errno));
 	exit(1);
      }
    if (tty)
@@ -527,7 +527,7 @@ fb_postinit(FB_Mode *mode)
 
   if (fb_fix.type != FB_TYPE_PACKED_PIXELS)
     {
-      fprintf(stderr,"can handle only packed pixel frame buffers\n");
+      CRIT("can handle only packed pixel frame buffers");
       fb_cleanup();
       exit(1);
     }
