@@ -15,6 +15,9 @@ struct _Evas_Engine_Info_Quartz
    struct {
       CGContextRef context;
    } info;
+
+   /* non-blocking or blocking mode */
+   Evas_Engine_Render_Mode render_mode;
 };
 
 #endif
