@@ -44,6 +44,7 @@ eng_info(Evas *e)
    info = calloc(1, sizeof(Evas_Engine_Info_Quartz));
    if (!info) return NULL;
    info->magic.magic = rand();
+   info->render_mode = EVAS_RENDER_MODE_BLOCKING;
    return info;
 }
 
@@ -732,10 +733,14 @@ eng_image_load(void *data, const char *file, const char *key, int *error, Evas_I
    // I can't figure out what to set it to, even trying to follow through the core code.
    // Also, none of the other engines set it.
 
-   if (error) *error = 0;
-   if (!im) return NULL;
+   if (!im)
+     {
+	*error = EVAS_LOAD_ERROR_RESOURCE_ALLOCATION_FAILED;
+	return NULL;
+     }
 
-   im->im = (RGBA_Image *)evas_common_load_image_from_file(file, key, lo);
+   *error = EVAS_LOAD_ERROR_NONE;
+   im->im = (RGBA_Image *)evas_common_load_image_from_file(file, key, lo, error);
    if (!im->im)
    {
       free(im);
