@@ -23,6 +23,9 @@ struct _Evas_Engine_Info_Software_Gdi
       unsigned int layered    : 1;
       unsigned int fullscreen : 1;
    } info;
+
+   /* non-blocking or blocking mode */
+   Evas_Engine_Render_Mode render_mode;
 };
 
 
