@@ -327,16 +327,21 @@ evas_software_xcb_x_output_buffer_new(xcb_connection_t *c,
 
                            /* XErrorHandler ph; */
                            /* EventHandlers eh; */
-
-                           free(xcb_get_input_focus_reply(c, xcb_get_input_focus_unchecked(c), NULL));
-                           _xcb_err = 0;
-                           /* ph = XSetErrorHandler((XErrorHandler) */
-                           /* x_output_tmp_x_err); */
+                            
+                           if (try_shm == 2) // only needed during testing
+                             {
+                               free(xcb_get_input_focus_reply(c, xcb_get_input_focus_unchecked(c), NULL));
+                               _xcb_err = 0;
+                               /* ph = XSetErrorHandler((XErrorHandler) */
+                             }
                            xcb_shm_attach(c,
                                           xcbob->shm_info->shmseg,
                                           xcbob->shm_info->shmid, 0);
-                           free(xcb_get_input_focus_reply(c, xcb_get_input_focus_unchecked(c), NULL));
-                           /* XSetErrorHandler((XErrorHandler)ph); */
+                           if (try_shm == 2) // only needed during testing
+                             {
+                               free(xcb_get_input_focus_reply(c, xcb_get_input_focus_unchecked(c), NULL));
+                               /* XSetErrorHandler((XErrorHandler)ph); */
+                             }
                            if (!_xcb_err)
                              {
 				 xcbob->bpl = xcbob->image->stride;
@@ -395,7 +400,6 @@ evas_software_xcb_x_output_buffer_free(Xcb_Output_Buffer *xcbob,
                                          xcb_get_input_focus_unchecked(xcbob->connection),
                                          NULL));
 	xcb_shm_detach(xcbob->connection, xcbob->shm_info->shmseg);
-	free(xcbob->image->data);
 	xcb_image_destroy(xcbob->image);
 	shmdt(xcbob->shm_info->shmaddr);
 	shmctl(xcbob->shm_info->shmid, IPC_RMID, 0);
@@ -424,7 +428,7 @@ evas_software_xcb_x_output_buffer_paste(Xcb_Output_Buffer    *xcbob,
                           xcbob->image, *xcbob->shm_info,
                           0, 0,
                           x, y,
-                          xcbob->image->width, xcbob->image->height,
+                          xcbob->w, xcbob->h,
                           0);
 	if (sync)
           free(xcb_get_input_focus_reply(xcbob->connection,
